"use strict";

var _react = _interopRequireDefault(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _angular = _interopRequireDefault(require("angular"));

require("angular-mocks");

var _ = require("../");

var _renderer = require("../renderer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

jest.mock('../renderer.js', function () {
  const original = require.requireActual('../renderer.js');

  return Object.assign({}, original, {
    createRenderer: jest.fn(original.createRenderer)
  });
});
afterEach(function () {
  _renderer.createRenderer.mockClear();
});
afterAll(function () {
  jest.unmock('../renderer.js');
});
describe('toComponent', function () {
  test('creates a function', function () {
    const decorators = [];
    const createComponent = (0, _.toComponent)(decorators);
    expect(typeof createComponent).toBe('function');
  });
  test('use empty array if no decorators are passed', function () {
    const createComponent = (0, _.toComponent)();
    expect(createComponent._decorators).toHaveLength(0);
  });
  describe('createComponent', function () {
    test('returns correct AngularJS component', function () {
      const decorators = [];

      const Component = function (props) {
        return _react.default.createElement("span", null, props.title);
      };

      const bindings = {
        title: '<'
      };
      const ngComponent = (0, _.toComponent)(decorators)(Component, bindings);
      expect(typeof ngComponent.controller).toBe('function');
      expect(ngComponent.bindings).toEqual(bindings);
    });
    test('use empty object if no bindings are passed', function () {
      const decorators = [];

      const Component = function () {
        return _react.default.createElement("span", null, "MyTitle");
      };

      const ngComponent = (0, _.toComponent)(decorators)(Component);
      expect(ngComponent.bindings).toEqual({});
    });
    describe('Controller', function () {
      test('resolves decorators correctly', function () {
        const decorators = [function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-1"
            }, render());
          };
        }, function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-2"
            }, render());
          };
        }];

        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const bindings = {
          title: '<'
        };
        const $element = [document.createElement('div')];
        const $injectInvoke = jest.fn(function (factory) {
          return factory();
        });
        const $injector = {
          invoke: $injectInvoke
        };

        const _toComponent = (0, _.toComponent)(decorators)(Component, bindings),
              Controller = _toComponent.controller;

        const ctrl = new Controller($element, $injector);
        expect($injectInvoke.mock.calls).toHaveLength(decorators.length);

        for (let i = 0, l = decorators.length; i < l; i++) {
          const call = $injectInvoke.mock.calls[i];
          const decorator = decorators[i];
          expect(call).toEqual([decorator]);
        }
      });
      test('creates correct WrappedComponent', function () {
        const decorators = [function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-1"
            }, render());
          };
        }, function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-2"
            }, render());
          };
        }];

        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const bindings = {
          title: '<'
        };
        const $element = [document.createElement('div')];

        const $injectInvoke = function (factory) {
          return factory();
        };

        const $injector = {
          invoke: $injectInvoke
        };

        const _toComponent2 = (0, _.toComponent)(decorators)(Component, bindings),
              Controller = _toComponent2.controller;

        const ctrl = new Controller($element, $injector);
        const WrappedComponent = ctrl.WrappedComponent;

        const tree = _reactTestRenderer.default.create(_react.default.createElement(WrappedComponent, {
          title: "MyTitle"
        })).toJSON();

        expect(tree).toMatchSnapshot();
      });
      test('creates renderer', function () {
        const decorators = [function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-1"
            }, render());
          };
        }, function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-2"
            }, render());
          };
        }];

        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const bindings = {
          title: '<'
        };
        const element = document.createElement('div');
        const $element = [element];

        const $injectInvoke = function (factory) {
          return factory();
        };

        const $injector = {
          invoke: $injectInvoke
        };

        const _toComponent3 = (0, _.toComponent)(decorators)(Component, bindings),
              Controller = _toComponent3.controller;

        const ctrl = new Controller($element, $injector);
        const WrappedComponent = ctrl.WrappedComponent;
        expect(_renderer.createRenderer.mock.calls).toHaveLength(1);
        const call = _renderer.createRenderer.mock.calls[0];
        expect(call).toEqual([WrappedComponent, element]);
      });
      test('$onChanges pass props to render function correctly', function () {
        const decorators = [function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-1"
            }, render());
          };
        }, function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-2"
            }, render());
          };
        }];

        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const bindings = {
          title: '<'
        };
        const scope = {
          title: 'MyTitle',
          unknownProp: 'unknown'
        };
        const props = {
          title: scope.title
        };
        const element = document.createElement('div');
        const $element = [element];

        const $injectInvoke = function (factory) {
          return factory();
        };

        const $injector = {
          invoke: $injectInvoke
        };

        const _toComponent4 = (0, _.toComponent)(decorators)(Component, bindings),
              Controller = _toComponent4.controller;

        const ctrl = new Controller($element, $injector); // We manually bind the scope the the controller

        ctrl.title = scope.title;
        ctrl.unknownProp = scope.unknownProp; // Mock render function

        const render = jest.fn();
        ctrl.renderer.render = render;
        ctrl.$onChanges();
        expect(render.mock.calls).toHaveLength(1);
        expect(render.mock.calls[0]).toEqual([props]);
      });
      test('$onDestroy unmounts correctly', function () {
        const decorators = [function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-1"
            }, render());
          };
        }, function () {
          return function (render) {
            return _react.default.createElement("span", {
              className: "Decorators-2"
            }, render());
          };
        }];

        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const bindings = {
          title: '<'
        };
        const element = document.createElement('div');
        const $element = [element];

        const $injectInvoke = function (factory) {
          return factory();
        };

        const $injector = {
          invoke: $injectInvoke
        };

        const _toComponent5 = (0, _.toComponent)(decorators)(Component, bindings),
              Controller = _toComponent5.controller;

        const ctrl = new Controller($element, $injector); // Mock unmount function

        const unmount = jest.fn();
        ctrl.renderer.unmount = unmount;
        ctrl.$onDestroy();
        expect(unmount.mock.calls).toHaveLength(1);
      });
    });
    describe('AngularJS component', function () {
      const createComponentRenderer = function (template, componentName, component) {
        const componentNameInTemplate = componentName.replace(/[A-Z]/g, function (c) {
          return '-' + c.toLowerCase();
        });
        let $compile;
        let $scope;

        _angular.default.mock.module(function ($compileProvider) {
          $compileProvider.component(componentName, component);
        });

        _angular.default.mock.inject(function (_$compile_, _$rootScope_) {
          $compile = _$compile_;
          $scope = _$rootScope_.$new();
        });

        const render = function () {
          const wrapper = $compile(template)($scope);

          wrapper.toElement = function () {
            return wrapper.children()[0];
          };

          return wrapper;
        };

        return {
          $scope: $scope,
          render: render
        };
      };

      test('renders correctly without props', function () {
        const Component = function () {
          return _react.default.createElement("span", null, "MyComponent");
        };

        const ngComponent = (0, _.toComponent)()(Component, {});
        const renderer = createComponentRenderer('<react-component-without-props></react-component-without-props>', 'reactComponentWithoutProps', ngComponent);
        expect(renderer.render().toElement()).toMatchSnapshot();
      });
      test('renders correctly with props', function () {
        const Component = function (props) {
          return _react.default.createElement("span", null, props.title);
        };

        const ngComponent = (0, _.toComponent)()(Component, {
          title: '<'
        });
        const renderer = createComponentRenderer('<react-component-with-props title="title"></react-component-with-props>', 'reactComponentWithProps', ngComponent);
        const $scope = renderer.$scope;
        const wrapper = renderer.render();
        $scope.title = 'My 1st Title';
        $scope.$digest();
        expect(wrapper.toElement()).toMatchSnapshot();
        $scope.title = 'My 2nd Title';
        $scope.$digest();
        expect(wrapper.toElement()).toMatchSnapshot();
      });
    });
  });
});