"use strict";

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _renderer = require("../renderer");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

jest.mock('react-dom', function () {
  return {
    render: jest.fn(),
    unmountComponentAtNode: jest.fn()
  };
});
describe('createRenderer', function () {
  test('creates render and unmount function', function () {
    const Component = function () {
      return _react.default.createElement("span", null);
    };

    const container = document.createElement('div');
    const renderer = (0, _renderer.createRenderer)(Component, container);
    expect(typeof renderer).toBe('object');
    expect(typeof renderer.render).toBe('function');
    expect(typeof renderer.unmount).toBe('function');
  });
  describe('render', function () {
    afterEach(function () {
      _reactDom.default.render.mockClear();
    });
    test('renders correct component into container', function () {
      const Component = function () {
        return _react.default.createElement("span", null);
      };

      const container = document.createElement('div');
      const props = {
        title: 'MyTitle'
      };

      const _createRenderer = (0, _renderer.createRenderer)(Component, container),
            render = _createRenderer.render;

      render();
      expect(_reactDom.default.render.mock.calls).toHaveLength(1);
      const call = _reactDom.default.render.mock.calls[0];
      expect(call[0].type).toBe(Component);
      expect(call[1]).toBe(container);
    });
    test('renders correctly without props', function () {
      const Component = function () {
        return _react.default.createElement("span", null);
      };

      const container = document.createElement('div');

      const _createRenderer2 = (0, _renderer.createRenderer)(Component, container),
            render = _createRenderer2.render;

      render();
      const element = _reactDom.default.render.mock.calls[0][0];
      expect(element.props).toEqual({});
    });
    test('renders correctly with props', function () {
      const Component = function () {
        return _react.default.createElement("span", null);
      };

      const container = document.createElement('div');
      const props = {
        title: 'MyTitle'
      };

      const _createRenderer3 = (0, _renderer.createRenderer)(Component, container),
            render = _createRenderer3.render;

      render(props);
      const element = _reactDom.default.render.mock.calls[0][0];
      expect(element.props).toEqual(props);
    });
  });
  test('unmount function unmounts correctly', function () {
    const Component = function () {
      return _react.default.createElement("span", null);
    };

    const container = document.createElement('div');

    const _createRenderer4 = (0, _renderer.createRenderer)(Component, container),
          unmount = _createRenderer4.unmount;

    unmount(container);
    expect(_reactDom.default.unmountComponentAtNode.mock.calls).toHaveLength(1);
    const call = _reactDom.default.unmountComponentAtNode.mock.calls[0];
    expect(call[0]).toBe(container);

    _reactDom.default.render.mockClear();

    _reactDom.default.unmountComponentAtNode.mockClear();
  });
});